@@ -19,17 +19,16 @@ extern void	ShowDawn(void);
 #define FIRE_KEY	0x05
 #define PAUSE_KEY	0x06
 #define ABORT_KEY	0x07
-#define HALT_KEY	0x08
-#define UNHALT_KEY	0x09
 
 /* The controls structure */
 typedef struct {
-	SDLKey gPauseControl;
-	SDLKey gShieldControl;
-	SDLKey gThrustControl;
-	SDLKey gTurnRControl;
-	SDLKey gTurnLControl;
-	SDLKey gFireControl;
-	SDLKey gQuitControl;
+	SDL_Keycode gPauseControl;
+	SDL_Keycode gShieldControl;
+	SDL_Keycode gThrustControl;
+	SDL_Keycode gTurnRControl;
+	SDL_Keycode gTurnLControl;
+	SDL_Keycode gFireControl;
+	SDL_Keycode gQuitControl;
 } Controls;
 
+extern SDL_Keymod gToggleFullscreenMod;
